# Copyright (c) 2023, Shukant Pal and Contributors
# SPDX-License-Identifier: Apache-2.0

from enum import IntEnum


class KtxSupercmpScheme(IntEnum):
    """Enumerators identifying the supercompression scheme."""

    NONE = 0
    """No supercompression."""

    BASIS_LZ = 1
    """Basis LZ supercompression."""

    ZSTD = 2
    """ZStd supercompression."""

    ZLIB = 3
    """ZLIB supercompression."""



# Copyright (c) 2023, Shukant Pal and Contributors
# SPDX-License-Identifier: Apache-2.0

from .gl_internalformat import GlInternalformat
from .ktx_error_code import KtxErrorCode, KtxError
from .ktx_texture import KtxTexture, KtxVersionMismatchError
from .ktx_texture_create_flag_bits import KtxTextureCreateFlagBits
from .ktx_texture_create_info import KtxTextureCreateInfo
from .ktx_texture_create_storage import KtxTextureCreateStorage
from pyktx.native import ffi, lib


class KtxTexture1(KtxTexture):
    """Class representing a KTX version 1 format texture."""

    @staticmethod
    def create(create_info: KtxTextureCreateInfo, storage_allocation: KtxTextureCreateStorage) -> 'KtxTexture1':
        """Create a new empty KtxTexture1."""

        result = lib.PY_ktxTexture1_Create(create_info.gl_internal_format.value,
                                           create_info.vk_format.value,
                                           ffi.NULL,
                                           create_info.base_width,
                                           create_info.base_height,
                                           create_info.base_depth,
                                           create_info.num_dimensions,
                                           create_info.num_levels,
                                           create_info.num_layers,
                                           create_info.num_faces,
                                           create_info.is_array,
                                           create_info.generate_mipmaps,
                                           storage_allocation.value)

        if int(result.error) != KtxErrorCode.SUCCESS:
            raise KtxError('ktxTexture1_Create', KtxErrorCode(result.error))

        return KtxTexture1(result.texture)

    @staticmethod
    def create_from_named_file(filename: str, create_flags: int = KtxTextureCreateFlagBits.NO_FLAGS) -> 'KtxTexture1':
        """Create a KtxTexture1 from a named KTX file according to the file contents."""

        result = lib.PY_ktxTexture_CreateFromNamedFile(filename.encode("ascii"), int(create_flags))

        if int(result.error) != KtxErrorCode.SUCCESS:
            raise KtxError('ktxTexture1_CreateFromNamedFile', KtxErrorCode(result.error))

        texture = KtxTexture1(result.texture)

        if texture.class_id != 1:
            raise KtxVersionMismatchError('The provided file ' + filename + ' is not a KTX1 file')

        return texture

    @property
    def gl_format(self) -> int:
        """
        Format of the texture data, e.g. GL_RGB.

        You can find all OpenGL formats here: <https://registry.khronos.org/OpenGL/api/GL/glcorearb.h>
        """
        return lib.PY_ktxTexture1_get_glFormat(self._ptr)

    @property
    def gl_internalformat(self) -> int:
        """
        Internal format of the texture data. See GlInternalformat.

        You can find all OpenGL internal formats here: <https://registry.khronos.org/OpenGL/api/GL/glext.h>
        """

        return GlInternalformat(lib.PY_ktxTexture1_get_glInternalformat(self._ptr))

    @property
    def gl_baseinternalformat(self) -> int:
        """
        Base format of the texture data, e.g., GL_RGB.

        You can find all OpenGL formats here: <https://registry.khronos.org/OpenGL/api/GL/glcorearb.h>
        """

        return lib.PY_ktxTexture1_get_glBaseInternalformat(self._ptr)

    @property
    def gl_type(self) -> int:
        """
        Type of the texture data, e.g, GL_UNSIGNED_BYTE.

        You can find all OpenGL data types here: <https://registry.khronos.org/OpenGL/api/GL/glcorearb.h>
        """
        return lib.PY_ktxTexture1_get_glType(self._ptr)



# Copyright (c) 2023, Shukant Pal and Contributors
# SPDX-License-Identifier: Apache-2.0

from pathlib import Path
import os

__test_images__ = os.environ['KTX_IMAGES_DIR'] \
    if 'KTX_IMAGES_DIR' in os.environ \
    else str((Path(__file__) / Path('../../../../tests/testimages')).resolve())



# Copyright (c) 2023, Shukant Pal and Contributors
# SPDX-License-Identifier: Apache-2.0

from .ktx_astc_params import KtxAstcParams
from .ktx_basis_params import KtxBasisParams
from .ktx_error_code import KtxErrorCode, KtxError
from .ktx_supercmp_scheme import KtxSupercmpScheme
from .ktx_texture import KtxTexture, KtxVersionMismatchError
from .ktx_texture_create_flag_bits import KtxTextureCreateFlagBits
from .ktx_texture_create_info import KtxTextureCreateInfo
from .ktx_texture_create_storage import KtxTextureCreateStorage
from .ktx_transcode_fmt import KtxTranscodeFmt
from pyktx.native import ffi, lib
from typing import Union
from .vk_format import VkFormat


class KtxTexture2(KtxTexture):
    """Class representing a KTX version 2 format texture."""

    @staticmethod
    def create(create_info: KtxTextureCreateInfo, storage_allocation: KtxTextureCreateStorage) -> 'KtxTexture2':
        """Create a new empty KtxTexture2."""

        result = lib.PY_ktxTexture2_Create(0,
                                           create_info.vk_format.value,
                                           ffi.NULL,
                                           create_info.base_width,
                                           create_info.base_height,
                                           create_info.base_depth,
                                           create_info.num_dimensions,
                                           create_info.num_levels,
                                           create_info.num_layers,
                                           create_info.num_faces,
                                           create_info.is_array,
                                           create_info.generate_mipmaps,
                                           storage_allocation.value)

        if int(result.error) != KtxErrorCode.SUCCESS:
            raise KtxError('ktxTexture2_Create', KtxErrorCode(result.error))

        return KtxTexture2(result.texture)

    @staticmethod
    def create_from_named_file(filename: str,
                               create_flags: int = KtxTextureCreateFlagBits.LOAD_IMAGE_DATA_BIT) -> 'KtxTexture2':
        """Create a KtxTexture2 from a named KTX file."""

        result = lib.PY_ktxTexture_CreateFromNamedFile(filename.encode("ascii"), int(create_flags))

        if int(result.error) != KtxErrorCode.SUCCESS:
            raise KtxError('ktxTexture2_CreateFromNamedFile', KtxErrorCode(result.error))

        texture = KtxTexture2(result.texture)

        if texture.class_id != 2:
            raise KtxVersionMismatchError('The provided file ' + filename + ' is not a KTX2 file')

        return texture

    @property
    def vk_format(self) -> VkFormat:
        """VkFormat for texture."""

        return VkFormat(lib.PY_ktxTexture2_get_vkFormat(self._ptr))

    @property
    def supercompression_scheme(self) -> KtxSupercmpScheme:
        """The supercompression scheme used to compress the texture data."""

        return KtxSupercmpScheme(lib.PY_ktxTexture2_get_supercompressionScheme(self._ptr))

    @property
    def oetf(self) -> int:
        """The opto-electrical transfer function of the images."""

        return lib.ktxTexture2_GetOETF(self._ptr)

    @property
    def premultipled_alpha(self) -> bool:
        """Whether the RGB components have been premultiplied by the alpha component."""

        return lib.ktxTexture2_GetPremultipliedAlpha(self._ptr)

    @property
    def needs_transcoding(self) -> bool:
        """If the images are in a transcodable format."""

        return lib.ktxTexture2_NeedsTranscoding(self._ptr)

    def compress_astc(self, params: Union[int, KtxAstcParams]) -> None:
        """
        Encode and compress a ktx texture with uncompressed images to ASTC.

        The images are either encoded to ASTC block-compressed format. The
        encoded images replace the original images and the texture's fields
        including the dfd are modified to reflect the new state.

        Such textures can be directly uploaded to a GPU via a graphics API.
        """

        if isinstance(params, int):
            quality = params
            params = KtxAstcParams()
            params.quality_level = quality

        error = lib.PY_ktxTexture2_CompressAstcEx(self._ptr,
                                                  params.verbose,
                                                  params.thread_count,
                                                  int(params.block_dimension),
                                                  int(params.mode),
                                                  params.quality_level,
                                                  params.normal_map,
                                                  params.perceptual,
                                                  params.input_swizzle)

        if int(error) != KtxErrorCode.SUCCESS:
            raise KtxError('ktxTexture2_compressAstcEx', KtxErrorCode(error))

    def compress_basis(self, params: Union[int, KtxBasisParams]) -> None:
        """
        Supercompress a KTX2 texture with uncompressed images.

        The images are either encoded to ETC1S block-compressed format and supercompressed
        with Basis LZ or they are encoded to UASTC block-compressed format. UASTC format is
        selected by setting the uastc field of params to true. The encoded images replace
        the original images and the texture's fields including the DFD are modified to reflect
        the new state.  Such textures must be transcoded to a desired target block compressed
        format before they can be uploaded to a GPU via a graphics API.
        """

        if isinstance(params, int):
            quality = params
            params = KtxBasisParams()
            params.quality_level = quality

        error = lib.PY_ktxTexture2_CompressBasisEx(self._ptr,
                                                   params.uastc,
                                                   params.verbose,
                                                   params.no_sse,
                                                   params.thread_count,
                                                   params.compression_level,
                                                   params.quality_level,
                                                   params.max_endpoints,
                                                   params.endpoint_rdo_threshold,
                                                   params.max_selectors,
                                                   params.selector_rdo_threshold,
                                                   params.input_swizzle,
                                                   params.normal_map,
                                                   params.pre_swizzle,
                                                   params.separate_rg_to_rgb_a,
                                                   params.no_endpoint_rdo,
                                                   params.no_selector_rdo,
                                                   params.uastc_flags,
                                                   params.uastc_rdo,
                                                   params.uastc_rdo_quality_scalar,
                                                   params.uastc_rdo_dict_size,
                                                   params.uastc_rdo_max_smooth_block_error_scale,
                                                   params.uastc_rdo_max_smooth_block_std_dev,
                                                   params.uastc_rdo_dont_favor_simpler_modes,
                                                   params.uastc_rdo_no_multithreading)

        if int(error) != KtxErrorCode.SUCCESS:
            raise KtxError('ktxTexture2_CompressBasisEx', KtxErrorCode(error))

    def deflate_zstd(self, compression_level: int) -> None:
        """
        Deflate the data in a ktxTexture2 object using Zstandard.

        The texture's level_index, data_size, dfd and supercompression_scheme will all
        be updated after successful deflation to reflect the deflated data.
        """

        if not 1 <= compression_level <= 22:
            raise ValueError("compression_level must be between 1 and 22")

        error = lib.ktxTexture2_DeflateZstd(self._ptr, compression_level)

        if int(error) != KtxErrorCode.SUCCESS:
            raise KtxError('ktx2_DeflateZstd', KtxErrorCode(error))

    def transcode_basis(self, output_format: KtxTranscodeFmt, transcode_flags: int = 0) -> None:
        """
        Transcode a KTX2 texture with BasisLZ/ETC1S or UASTC images.

        If the texture contains BasisLZ supercompressed images, inflates them from back to
        ETC1S then transcodes them to the specified block-compressed format. If the texture
        contains UASTC images, inflates them, if they have been supercompressed with zstd, then
        transcodes then to the specified format, The transcoded images replace the original images
        and the texture's fields including the dfd are modified to reflect the new format.

        These types of textures must be transcoded to a desired target block-compressed format
        before they can be uploaded to a GPU via a graphics API.

        The following block compressed transcode targets (KtxTranscodeFmt) are available: ETC1_RGB,
        ETC2_RGBA, BC1_RGB, BC3_RGBA, BC4_R, BC5_RG, BC7_RGBA, PVRTC1_4_RGB, PVRTC1_4_RGBA,
        PVRTC2_4_RGB, PVRTC2_4_RGBA, ASTC_4x4_RGBA, ETC2_EAC_R11, ETC2_EAC_RG11, ETC and BC1_OR_3.

        ETC automatically selects between ETC1_RGB and ETC2_RGBA according to whether an alpha
        channel is available. BC1_OR_3 does likewise between BC1_RGB and BC3_RGBA. Note that if
        PVRTC1_4_RGBA or PVRTC2_4_RGBA is specified and there is no alpha channel PVRTC1_4_RGB
        or PVRTC2_4_RGB respectively will be selected.

        Transcoding to ATC & FXT1 formats is not supported by libktx as there are no equivalent Vulkan formats.

        The following uncompressed transcode targets are also available: RGBA32, RGB565, BGR565 and RGBA4444.
        """

        error = lib.ktxTexture2_TranscodeBasis(self._ptr, output_format, transcode_flags)

        if int(error) != KtxErrorCode.SUCCESS:
            raise KtxError('ktx2_TranscodeBasis', KtxErrorCode(error))



# Copyright (c) 2023, Shukant Pal and Contributors
# SPDX-License-Identifier: Apache-2.0

import os

LIBKTX_INSTALL_DIR = os.getenv("LIBKTX_INSTALL_DIR")
LIBKTX_LIB_DIR = os.getenv("LIBKTX_LIB_DIR")

if os.name == 'nt':
    if LIBKTX_INSTALL_DIR is None:
        LIBKTX_INSTALL_DIR = 'C:\\Program Files\\KTX-Software'
    if LIBKTX_LIB_DIR is None:
        LIBKTX_LIB_DIR = LIBKTX_INSTALL_DIR + '\\bin'
    os.add_dll_directory(os.path.normpath(LIBKTX_LIB_DIR))

from .gl_internalformat import *
from .ktx_astc_params import *
from .ktx_basis_params import *
from .ktx_error_code import *
from .ktx_hash_list import *
from .ktx_pack_astc_block_dimension import *
from .ktx_pack_astc_encoder_mode import *
from .ktx_pack_astc_quality_levels import *
from .ktx_pack_uastc_flag_bits import *
from .ktx_supercmp_scheme import *
from .ktx_texture import *
from .ktx_texture1 import *
from .ktx_texture2 import *
from .ktx_texture_create_flag_bits import *
from .ktx_texture_create_info import *
from .ktx_texture_create_storage import *
from .ktx_transcode_flag_bits import *
from .ktx_transcode_fmt import *
from .vk_format import *



# Copyright (c) 2023, Shukant Pal and Contributors
# SPDX-License-Identifier: Apache-2.0

from enum import IntEnum


class KtxTextureCreateFlagBits(IntEnum):
    """Flags for requesting services during creation."""

    NO_FLAGS = 0x00

    LOAD_IMAGE_DATA_BIT = 0x01
    """Load the images from the KTX source."""

    RAW_KVDATA_BIT = 0x02
    """Load the raw key-value data instead of creating a KtxHashList from it."""

    SKIP_KVDATA_BIT = 0x04
    """Skip any key-value data. This overrides the RAW_KVDATA_BIT."""



# Copyright (c) 2023, Shukant Pal and Contributors
# SPDX-License-Identifier: Apache-2.0

from pyktx import *
from tempfile import NamedTemporaryFile
from test_config import __test_images__
import os
import unittest


class TestKtxTexture1(unittest.TestCase):
    def test_create_from_named_file(self):
        test_ktx_file = os.path.join(__test_images__, 'etc1.ktx')
        texture = KtxTexture1.create_from_named_file(test_ktx_file)

        self.assertEqual(texture.gl_internalformat, GlInternalformat.ETC1_RGB8_OES)
        self.assertFalse(texture.is_array)
        self.assertFalse(texture.generate_mipmaps)
        self.assertEqual(texture.num_levels, 1)

    def test_write_to_named_file(self):
        test_ktx_file = os.path.join(__test_images__, 'etc2-rgb.ktx')
        copy_file = NamedTemporaryFile(delete=False)
        copy_file.close()

        texture = KtxTexture1.create_from_named_file(test_ktx_file, KtxTextureCreateFlagBits.LOAD_IMAGE_DATA_BIT)
        texture.write_to_named_file(copy_file.name)

        with open(test_ktx_file, 'rb') as original, open(copy_file.name, 'rb') as copy:
            self.assertEqual(original.read(), copy.read())

        os.unlink(copy_file.name)

    def test_write_to_memory(self):
        test_ktx_file = os.path.join(__test_images__, 'etc2-rgba1.ktx')
        texture = KtxTexture1.create_from_named_file(test_ktx_file, KtxTextureCreateFlagBits.LOAD_IMAGE_DATA_BIT)
        texture_bytes = texture.write_to_memory()

        with open(test_ktx_file, 'rb') as original:
            self.assertEqual(original.read(), texture_bytes)

    def test_get_data(self):
        test_ktx_file = os.path.join(__test_images__, 'etc2-rgba1.ktx')
        texture = KtxTexture1.create_from_named_file(test_ktx_file, KtxTextureCreateFlagBits.LOAD_IMAGE_DATA_BIT)

        data = texture.data()

        with open(test_ktx_file, 'rb') as file:
            level_0_size = texture.image_size(0)
            self.assertEqual(file.read()[-level_0_size:], data[:])

    def test_create(self):
        info = KtxTextureCreateInfo(
            gl_internal_format=GlInternalformat.COMPRESSED_RGBA_ASTC_4x4_KHR,
            base_width=10,
            base_height=10,
            base_depth=1)

        texture = KtxTexture1.create(info, KtxTextureCreateStorage.ALLOC)
        texture.set_image_from_memory(0, 0, 0, bytes(texture.data_size))




#******************************* Do not edit.  *****************************
# Automatically generated from vulkan_core.h version 287 by mkvkformatfiles.
#***************************************************************************

# Copyright 2015-2024 The Khronos Group Inc.
#
# SPDX-License-Identifier: Apache-2.0

from enum import IntEnum

class VkFormat(IntEnum):
    """Vulkan texture format constants."""

    VK_FORMAT_UNDEFINED = 0
    VK_FORMAT_R4G4_UNORM_PACK8 = 1
    VK_FORMAT_R4G4B4A4_UNORM_PACK16 = 2
    VK_FORMAT_B4G4R4A4_UNORM_PACK16 = 3
    VK_FORMAT_R5G6B5_UNORM_PACK16 = 4
    VK_FORMAT_B5G6R5_UNORM_PACK16 = 5
    VK_FORMAT_R5G5B5A1_UNORM_PACK16 = 6
    VK_FORMAT_B5G5R5A1_UNORM_PACK16 = 7
    VK_FORMAT_A1R5G5B5_UNORM_PACK16 = 8
    VK_FORMAT_R8_UNORM = 9
    VK_FORMAT_R8_SNORM = 10
    VK_FORMAT_R8_UINT = 13
    VK_FORMAT_R8_SINT = 14
    VK_FORMAT_R8_SRGB = 15
    VK_FORMAT_R8G8_UNORM = 16
    VK_FORMAT_R8G8_SNORM = 17
    VK_FORMAT_R8G8_UINT = 20
    VK_FORMAT_R8G8_SINT = 21
    VK_FORMAT_R8G8_SRGB = 22
    VK_FORMAT_R8G8B8_UNORM = 23
    VK_FORMAT_R8G8B8_SNORM = 24
    VK_FORMAT_R8G8B8_UINT = 27
    VK_FORMAT_R8G8B8_SINT = 28
    VK_FORMAT_R8G8B8_SRGB = 29
    VK_FORMAT_B8G8R8_UNORM = 30
    VK_FORMAT_B8G8R8_SNORM = 31
    VK_FORMAT_B8G8R8_UINT = 34
    VK_FORMAT_B8G8R8_SINT = 35
    VK_FORMAT_B8G8R8_SRGB = 36
    VK_FORMAT_R8G8B8A8_UNORM = 37
    VK_FORMAT_R8G8B8A8_SNORM = 38
    VK_FORMAT_R8G8B8A8_UINT = 41
    VK_FORMAT_R8G8B8A8_SINT = 42
    VK_FORMAT_R8G8B8A8_SRGB = 43
    VK_FORMAT_B8G8R8A8_UNORM = 44
    VK_FORMAT_B8G8R8A8_SNORM = 45
    VK_FORMAT_B8G8R8A8_UINT = 48
    VK_FORMAT_B8G8R8A8_SINT = 49
    VK_FORMAT_B8G8R8A8_SRGB = 50
    VK_FORMAT_A8B8G8R8_UNORM_PACK32 = 51
    VK_FORMAT_A8B8G8R8_SNORM_PACK32 = 52
    VK_FORMAT_A8B8G8R8_UINT_PACK32 = 55
    VK_FORMAT_A8B8G8R8_SINT_PACK32 = 56
    VK_FORMAT_A8B8G8R8_SRGB_PACK32 = 57
    VK_FORMAT_A2R10G10B10_UNORM_PACK32 = 58
    VK_FORMAT_A2R10G10B10_SNORM_PACK32 = 59
    VK_FORMAT_A2R10G10B10_UINT_PACK32 = 62
    VK_FORMAT_A2R10G10B10_SINT_PACK32 = 63
    VK_FORMAT_A2B10G10R10_UNORM_PACK32 = 64
    VK_FORMAT_A2B10G10R10_SNORM_PACK32 = 65
    VK_FORMAT_A2B10G10R10_UINT_PACK32 = 68
    VK_FORMAT_A2B10G10R10_SINT_PACK32 = 69
    VK_FORMAT_R16_UNORM = 70
    VK_FORMAT_R16_SNORM = 71
    VK_FORMAT_R16_UINT = 74
    VK_FORMAT_R16_SINT = 75
    VK_FORMAT_R16_SFLOAT = 76
    VK_FORMAT_R16G16_UNORM = 77
    VK_FORMAT_R16G16_SNORM = 78
    VK_FORMAT_R16G16_UINT = 81
    VK_FORMAT_R16G16_SINT = 82
    VK_FORMAT_R16G16_SFLOAT = 83
    VK_FORMAT_R16G16B16_UNORM = 84
    VK_FORMAT_R16G16B16_SNORM = 85
    VK_FORMAT_R16G16B16_UINT = 88
    VK_FORMAT_R16G16B16_SINT = 89
    VK_FORMAT_R16G16B16_SFLOAT = 90
    VK_FORMAT_R16G16B16A16_UNORM = 91
    VK_FORMAT_R16G16B16A16_SNORM = 92
    VK_FORMAT_R16G16B16A16_UINT = 95
    VK_FORMAT_R16G16B16A16_SINT = 96
    VK_FORMAT_R16G16B16A16_SFLOAT = 97
    VK_FORMAT_R32_UINT = 98
    VK_FORMAT_R32_SINT = 99
    VK_FORMAT_R32_SFLOAT = 100
    VK_FORMAT_R32G32_UINT = 101
    VK_FORMAT_R32G32_SINT = 102
    VK_FORMAT_R32G32_SFLOAT = 103
    VK_FORMAT_R32G32B32_UINT = 104
    VK_FORMAT_R32G32B32_SINT = 105
    VK_FORMAT_R32G32B32_SFLOAT = 106
    VK_FORMAT_R32G32B32A32_UINT = 107
    VK_FORMAT_R32G32B32A32_SINT = 108
    VK_FORMAT_R32G32B32A32_SFLOAT = 109
    VK_FORMAT_R64_UINT = 110
    VK_FORMAT_R64_SINT = 111
    VK_FORMAT_R64_SFLOAT = 112
    VK_FORMAT_R64G64_UINT = 113
    VK_FORMAT_R64G64_SINT = 114
    VK_FORMAT_R64G64_SFLOAT = 115
    VK_FORMAT_R64G64B64_UINT = 116
    VK_FORMAT_R64G64B64_SINT = 117
    VK_FORMAT_R64G64B64_SFLOAT = 118
    VK_FORMAT_R64G64B64A64_UINT = 119
    VK_FORMAT_R64G64B64A64_SINT = 120
    VK_FORMAT_R64G64B64A64_SFLOAT = 121
    VK_FORMAT_B10G11R11_UFLOAT_PACK32 = 122
    VK_FORMAT_E5B9G9R9_UFLOAT_PACK32 = 123
    VK_FORMAT_D16_UNORM = 124
    VK_FORMAT_X8_D24_UNORM_PACK32 = 125
    VK_FORMAT_D32_SFLOAT = 126
    VK_FORMAT_S8_UINT = 127
    VK_FORMAT_D16_UNORM_S8_UINT = 128
    VK_FORMAT_D24_UNORM_S8_UINT = 129
    VK_FORMAT_D32_SFLOAT_S8_UINT = 130
    VK_FORMAT_BC1_RGB_UNORM_BLOCK = 131
    VK_FORMAT_BC1_RGB_SRGB_BLOCK = 132
    VK_FORMAT_BC1_RGBA_UNORM_BLOCK = 133
    VK_FORMAT_BC1_RGBA_SRGB_BLOCK = 134
    VK_FORMAT_BC2_UNORM_BLOCK = 135
    VK_FORMAT_BC2_SRGB_BLOCK = 136
    VK_FORMAT_BC3_UNORM_BLOCK = 137
    VK_FORMAT_BC3_SRGB_BLOCK = 138
    VK_FORMAT_BC4_UNORM_BLOCK = 139
    VK_FORMAT_BC4_SNORM_BLOCK = 140
    VK_FORMAT_BC5_UNORM_BLOCK = 141
    VK_FORMAT_BC5_SNORM_BLOCK = 142
    VK_FORMAT_BC6H_UFLOAT_BLOCK = 143
    VK_FORMAT_BC6H_SFLOAT_BLOCK = 144
    VK_FORMAT_BC7_UNORM_BLOCK = 145
    VK_FORMAT_BC7_SRGB_BLOCK = 146
    VK_FORMAT_ETC2_R8G8B8_UNORM_BLOCK = 147
    VK_FORMAT_ETC2_R8G8B8_SRGB_BLOCK = 148
    VK_FORMAT_ETC2_R8G8B8A1_UNORM_BLOCK = 149
    VK_FORMAT_ETC2_R8G8B8A1_SRGB_BLOCK = 150
    VK_FORMAT_ETC2_R8G8B8A8_UNORM_BLOCK = 151
    VK_FORMAT_ETC2_R8G8B8A8_SRGB_BLOCK = 152
    VK_FORMAT_EAC_R11_UNORM_BLOCK = 153
    VK_FORMAT_EAC_R11_SNORM_BLOCK = 154
    VK_FORMAT_EAC_R11G11_UNORM_BLOCK = 155
    VK_FORMAT_EAC_R11G11_SNORM_BLOCK = 156
    VK_FORMAT_ASTC_4x4_UNORM_BLOCK = 157
    VK_FORMAT_ASTC_4x4_SRGB_BLOCK = 158
    VK_FORMAT_ASTC_5x4_UNORM_BLOCK = 159
    VK_FORMAT_ASTC_5x4_SRGB_BLOCK = 160
    VK_FORMAT_ASTC_5x5_UNORM_BLOCK = 161
    VK_FORMAT_ASTC_5x5_SRGB_BLOCK = 162
    VK_FORMAT_ASTC_6x5_UNORM_BLOCK = 163
    VK_FORMAT_ASTC_6x5_SRGB_BLOCK = 164
    VK_FORMAT_ASTC_6x6_UNORM_BLOCK = 165
    VK_FORMAT_ASTC_6x6_SRGB_BLOCK = 166
    VK_FORMAT_ASTC_8x5_UNORM_BLOCK = 167
    VK_FORMAT_ASTC_8x5_SRGB_BLOCK = 168
    VK_FORMAT_ASTC_8x6_UNORM_BLOCK = 169
    VK_FORMAT_ASTC_8x6_SRGB_BLOCK = 170
    VK_FORMAT_ASTC_8x8_UNORM_BLOCK = 171
    VK_FORMAT_ASTC_8x8_SRGB_BLOCK = 172
    VK_FORMAT_ASTC_10x5_UNORM_BLOCK = 173
    VK_FORMAT_ASTC_10x5_SRGB_BLOCK = 174
    VK_FORMAT_ASTC_10x6_UNORM_BLOCK = 175
    VK_FORMAT_ASTC_10x6_SRGB_BLOCK = 176
    VK_FORMAT_ASTC_10x8_UNORM_BLOCK = 177
    VK_FORMAT_ASTC_10x8_SRGB_BLOCK = 178
    VK_FORMAT_ASTC_10x10_UNORM_BLOCK = 179
    VK_FORMAT_ASTC_10x10_SRGB_BLOCK = 180
    VK_FORMAT_ASTC_12x10_UNORM_BLOCK = 181
    VK_FORMAT_ASTC_12x10_SRGB_BLOCK = 182
    VK_FORMAT_ASTC_12x12_UNORM_BLOCK = 183
    VK_FORMAT_ASTC_12x12_SRGB_BLOCK = 184
    VK_FORMAT_G8B8G8R8_422_UNORM = 1000156000
    VK_FORMAT_B8G8R8G8_422_UNORM = 1000156001
    VK_FORMAT_R10X6_UNORM_PACK16 = 1000156007
    VK_FORMAT_R10X6G10X6_UNORM_2PACK16 = 1000156008
    VK_FORMAT_R10X6G10X6B10X6A10X6_UNORM_4PACK16 = 1000156009
    VK_FORMAT_G10X6B10X6G10X6R10X6_422_UNORM_4PACK16 = 1000156010
    VK_FORMAT_B10X6G10X6R10X6G10X6_422_UNORM_4PACK16 = 1000156011
    VK_FORMAT_R12X4_UNORM_PACK16 = 1000156017
    VK_FORMAT_R12X4G12X4_UNORM_2PACK16 = 1000156018
    VK_FORMAT_R12X4G12X4B12X4A12X4_UNORM_4PACK16 = 1000156019
    VK_FORMAT_G12X4B12X4G12X4R12X4_422_UNORM_4PACK16 = 1000156020
    VK_FORMAT_B12X4G12X4R12X4G12X4_422_UNORM_4PACK16 = 1000156021
    VK_FORMAT_G16B16G16R16_422_UNORM = 1000156027
    VK_FORMAT_B16G16R16G16_422_UNORM = 1000156028
    VK_FORMAT_A4R4G4B4_UNORM_PACK16 = 1000340000
    VK_FORMAT_A4B4G4R4_UNORM_PACK16 = 1000340001
    VK_FORMAT_ASTC_4x4_SFLOAT_BLOCK = 1000066000
    VK_FORMAT_ASTC_5x4_SFLOAT_BLOCK = 1000066001
    VK_FORMAT_ASTC_5x5_SFLOAT_BLOCK = 1000066002
    VK_FORMAT_ASTC_6x5_SFLOAT_BLOCK = 1000066003
    VK_FORMAT_ASTC_6x6_SFLOAT_BLOCK = 1000066004
    VK_FORMAT_ASTC_8x5_SFLOAT_BLOCK = 1000066005
    VK_FORMAT_ASTC_8x6_SFLOAT_BLOCK = 1000066006
    VK_FORMAT_ASTC_8x8_SFLOAT_BLOCK = 1000066007
    VK_FORMAT_ASTC_10x5_SFLOAT_BLOCK = 1000066008
    VK_FORMAT_ASTC_10x6_SFLOAT_BLOCK = 1000066009
    VK_FORMAT_ASTC_10x8_SFLOAT_BLOCK = 1000066010
    VK_FORMAT_ASTC_10x10_SFLOAT_BLOCK = 1000066011
    VK_FORMAT_ASTC_12x10_SFLOAT_BLOCK = 1000066012
    VK_FORMAT_ASTC_12x12_SFLOAT_BLOCK = 1000066013
    VK_FORMAT_PVRTC1_2BPP_UNORM_BLOCK_IMG = 1000054000
    VK_FORMAT_PVRTC1_4BPP_UNORM_BLOCK_IMG = 1000054001
    VK_FORMAT_PVRTC2_2BPP_UNORM_BLOCK_IMG = 1000054002
    VK_FORMAT_PVRTC2_4BPP_UNORM_BLOCK_IMG = 1000054003
    VK_FORMAT_PVRTC1_2BPP_SRGB_BLOCK_IMG = 1000054004
    VK_FORMAT_PVRTC1_4BPP_SRGB_BLOCK_IMG = 1000054005
    VK_FORMAT_PVRTC2_2BPP_SRGB_BLOCK_IMG = 1000054006
    VK_FORMAT_PVRTC2_4BPP_SRGB_BLOCK_IMG = 1000054007
    VK_FORMAT_ASTC_3x3x3_UNORM_BLOCK_EXT = 1000288000
    VK_FORMAT_ASTC_3x3x3_SRGB_BLOCK_EXT = 1000288001
    VK_FORMAT_ASTC_3x3x3_SFLOAT_BLOCK_EXT = 1000288002
    VK_FORMAT_ASTC_4x3x3_UNORM_BLOCK_EXT = 1000288003
    VK_FORMAT_ASTC_4x3x3_SRGB_BLOCK_EXT = 1000288004
    VK_FORMAT_ASTC_4x3x3_SFLOAT_BLOCK_EXT = 1000288005
    VK_FORMAT_ASTC_4x4x3_UNORM_BLOCK_EXT = 1000288006
    VK_FORMAT_ASTC_4x4x3_SRGB_BLOCK_EXT = 1000288007
    VK_FORMAT_ASTC_4x4x3_SFLOAT_BLOCK_EXT = 1000288008
    VK_FORMAT_ASTC_4x4x4_UNORM_BLOCK_EXT = 1000288009
    VK_FORMAT_ASTC_4x4x4_SRGB_BLOCK_EXT = 1000288010
    VK_FORMAT_ASTC_4x4x4_SFLOAT_BLOCK_EXT = 1000288011
    VK_FORMAT_ASTC_5x4x4_UNORM_BLOCK_EXT = 1000288012
    VK_FORMAT_ASTC_5x4x4_SRGB_BLOCK_EXT = 1000288013
    VK_FORMAT_ASTC_5x4x4_SFLOAT_BLOCK_EXT = 1000288014
    VK_FORMAT_ASTC_5x5x4_UNORM_BLOCK_EXT = 1000288015
    VK_FORMAT_ASTC_5x5x4_SRGB_BLOCK_EXT = 1000288016
    VK_FORMAT_ASTC_5x5x4_SFLOAT_BLOCK_EXT = 1000288017
    VK_FORMAT_ASTC_5x5x5_UNORM_BLOCK_EXT = 1000288018
    VK_FORMAT_ASTC_5x5x5_SRGB_BLOCK_EXT = 1000288019
    VK_FORMAT_ASTC_5x5x5_SFLOAT_BLOCK_EXT = 1000288020
    VK_FORMAT_ASTC_6x5x5_UNORM_BLOCK_EXT = 1000288021
    VK_FORMAT_ASTC_6x5x5_SRGB_BLOCK_EXT = 1000288022
    VK_FORMAT_ASTC_6x5x5_SFLOAT_BLOCK_EXT = 1000288023
    VK_FORMAT_ASTC_6x6x5_UNORM_BLOCK_EXT = 1000288024
    VK_FORMAT_ASTC_6x6x5_SRGB_BLOCK_EXT = 1000288025
    VK_FORMAT_ASTC_6x6x5_SFLOAT_BLOCK_EXT = 1000288026
    VK_FORMAT_ASTC_6x6x6_UNORM_BLOCK_EXT = 1000288027
    VK_FORMAT_ASTC_6x6x6_SRGB_BLOCK_EXT = 1000288028
    VK_FORMAT_ASTC_6x6x6_SFLOAT_BLOCK_EXT = 1000288029
    VK_FORMAT_R16G16_SFIXED5_NV = 1000464000
    VK_FORMAT_A1B5G5R5_UNORM_PACK16_KHR = 1000470000
    VK_FORMAT_A8_UNORM_KHR = 1000470001




# Copyright (c) 2023, Shukant Pal and Contributors
# SPDX-License-Identifier: Apache-2.0

from enum import IntEnum


class KtxErrorCode(IntEnum):
    """Error codes thrown by library functions."""

    SUCCESS = 0
    """Operation was successful."""

    FILE_DATA_ERROR = 1
    """The data in the file is inconsistent with the spec."""

    FILE_ISPIPE = 2
    """The file is a pipe or named pipe."""

    FILE_OPEN_FAILED = 3
    """The target file could not be opened."""

    FILE_OVERFLOW = 4
    """The operation would exceed the max file size."""

    FILE_READ_ERROR = 5
    """An error occurred while reading from the file."""

    FILE_SEEK_ERROR = 6
    """An error occurred while seeking in the file."""

    FILE_UNEXPECTED_EOF = 7
    """File does not have enough data to satisfy request."""

    FILE_WRITE_ERROR = 8
    """An error occurred while writing to the file."""

    GL_ERROR = 9
    """GL operations resulted in an error."""

    INVALID_OPERATION = 10
    """The operation is not allowed in the current state."""

    INVALID_VALUE = 11
    """A parameter value was not valid."""

    NOT_FOUND = 12
    """Requested key was not found"""

    OUT_OF_MEMORY = 13
    """Not enough memory to complete the operation."""

    TRANSCODE_FAILED = 14
    """Transcoding of block compressed texture failed."""

    UNKNOWN_FILE_FORMAT = 15
    """The file not a KTX file."""

    UNSUPPORTED_TEXTURE_TYPE = 16
    """The KTX file specifies an unsupported texture type."""

    UNSUPPORTED_FEATURE = 17
    """Feature not included in in-use library or not yet implemented."""

    LIBRARY_NOT_LINKED = 18
    """Library dependency (OpenGL or Vulkan) not linked into application."""

    DECOMPRESS_LENGTH_ERROR = 19
    """Decompressed byte count does not match expected byte size."""

    DECOMPRESS_CHECKSUM_ERROR = 20
    """Checksum mismatch when decompressing."""

    ERROR_MAX_ENUM = LIBRARY_NOT_LINKED
    """For safety checks."""


class KtxError(Exception):
    """Error thrown when native operation does not succeed."""

    invocation: str
    """The C library function called."""

    code: KtxErrorCode
    """The error code returned by libktx."""

    def __init__(self, invocation: str, code: KtxErrorCode):
        self.invocation = invocation
        self.code = code

    def __str__(self):
        return str(self.invocation) + " returned with " + str(self.code)



# Copyright (c) 2023, Shukant Pal and Contributors
# SPDX-License-Identifier: Apache-2.0

from pyktx import *
from test_config import __test_images__
import unittest


class TestKtxTexture(unittest.TestCase):
    def test_kv_data(self):
        test_ktx_file = os.path.join(__test_images__, 'astc_ldr_4x4_FlightHelmet_baseColor.ktx2')
        texture = KtxTexture2.create_from_named_file(test_ktx_file)

        self.assertEqual(texture.kv_data.find_value('KTXorientation'), b'rd\x00')
        self.assertEqual(texture.kv_data.copy(), {
            'KTXorientation': b'rd\x00',
            'KTXwriter': b'toktx v4.0.__default__ / libktx v4.0.__default__\x00',
            'KTXwriterScParams': b'--encode astc --astc_blk_d 4x4\x00',
        })

        for key in ['KTXorientation', 'KTXwriter', 'KTXwriterScParams']:
            texture.kv_data.delete_kv_pair(key)
        texture.kv_data.add_kv_pair('KTXwriter', b'pyktx v4.0.__default__ / libktx v4.0.__default__\x00')

        self.assertEqual(texture.kv_data.find_value('KTXwriter'),
                         b'pyktx v4.0.__default__ / libktx v4.0.__default__\x00')
